/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
*/
/**
 * beat.accounts.dialogs.js
 * Implementation of experiment-related dialogs
 */
if (beat === undefined) var beat = {};
if (beat.accounts === undefined) beat.accounts = {};
if (beat.accounts.dialogs === undefined) beat.accounts.dialogs = {};

/**
 * Implements a modal dialog to renew a user account.
 * The user is presented with a small modal window that will allow
 * him to certify the wish to renew his account. It is possible to cancel
 * the action at any moment by pressing "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API for renewing the account
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_renew_account = function(dialog_id, api_url, redirect_url) {

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Revalidate',
      click: function() {

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "PUT",
          url: api_url,
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);

            $(this).dialog("close");
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog to change supervisor.
 * The user is presented with a small modal window that will allow
 * him to add the new supervisor username. It is possible to cancel the action at
 * any moment by pressing "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   old_supervisor_username: previous supervisor username
 *   api_url (str): The URL towards the API for changing the supervisor
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_change_supervisor = function(dialog_id, old_supervisor_username, api_url, redirect_url) {

  var split_api_url = api_url.split(old_supervisor_username);

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Request',
      click: function() {

      var new_supervisor_username = $('#' + dialog_id + ' #new_supervisor_username')[0].value;

      var name = $('#' + dialog_id + ' input')[0].value.trim();

        if (name.length == 0)
        {
          alert("You can't enter an empty supervisor username!");
          return;
        }

        if (new_supervisor_username == old_supervisor_username)
        {
          alert("The new supervisor username is the same as the previous one!");
          return;
        }

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "POST",
          url: split_api_url[0] + new_supervisor_username + split_api_url[1],
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Initialise the dialog content
  $('#' + dialog_id + ' input')[0].value = old_supervisor_username;

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog to add a supervisor.
 * The user is presented with a small modal window that will allow
 * him to add to set the supervisor username. It is possible to cancel the action at
 * any moment by pressing "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API for adding a supervisor
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_add_supervisor = function(dialog_id, api_url, redirect_url) {

  var split_api_url = api_url.split("nosupervisor");

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Request',
      click: function() {

      var new_supervisor_username = $('#' + dialog_id + ' #new_supervisor_username')[0].value;

      var name = $('#' + dialog_id + ' input')[0].value.trim();

        if (name.length == 0)
        {
          alert("You can't enter an empty supervisor username!");
          return;
        }

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "POST",
          url: split_api_url[0] + new_supervisor_username + split_api_url[1],
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Initialise the dialog content
  $('#' + dialog_id + ' input')[0].value = '';

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog for the supervisor to validate a supervisee
 * account. It is possible to cancel the action at any moment by pressing
 * "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API to validate a supervisee
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_validate_supervisee = function(dialog_id, api_url, redirect_url) {

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Accept',
      click: function() {

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "PUT",
          url: api_url,
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);

            $(this).dialog("close");
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog for the supervisor to remove a supervisee
 * account. It is possible to cancel the action at any moment by pressing
 * "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API to remove a supervisee
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_remove_supervisee = function(dialog_id, api_url, redirect_url) {

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Reject',
      click: function() {

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "PUT",
          url: api_url,
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);

            $(this).dialog("close");
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog for the supervisor to remove a supervisee
 * account. It is possible to cancel the action at any moment by pressing
 * "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API to remove a supervisee
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_remove_supervisor_supervisee = function(dialog_id, api_url, redirect_url) {

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Reject',
      click: function() {

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "PUT",
          url: api_url,
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);

            $(this).dialog("close");
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog for the admin to become supervisor
 * account. It is possible to cancel the action at any moment by pressing
 * "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API to remove a supervisee
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_admin_set_supervisor_mode = function(dialog_id, api_url, redirect_url) {

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Set',
      click: function() {

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "PUT",
          url: api_url,
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);

            $(this).dialog("close");
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $('#toggle-supervisor-event').prop('cancelaction', false);
        $('#toggle-supervisor-event').bootstrapToggle('off');
        $(this).dialog("close");
      }
    }
    ]
  });

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog for the admin to unset supervisor mode
 * account. It is possible to cancel the action at any moment by pressing
 * "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API to remove a supervisee
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_admin_remove_supervisor_mode = function(dialog_id, api_url, redirect_url) {

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Remove',
      click: function() {

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "PUT",
          url: api_url,
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);

            $(this).dialog("close");
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $('#toggle-supervisor-event').prop('cancelaction', false);
        $('#toggle-supervisor-event').bootstrapToggle('on');
        $(this).dialog("close");
      }
    }
    ]
  });

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}


/**
 * Uses bootstrap3 to display a modal window to grant supervisor access to user. The user is
 * presented with a "typeahead" input box so they can choose a proper username
 * The typeahead auto-completion works for the list of specific user (valid users)
 *
 * Parameters:
 *
 *   url (String): A string with the URL to retrieve specific users
 *     names and to grant access
 *   username (String): The name of the user requesting the operation
 *
 **/

beat.accounts.new_supervisor_access = function (list_url, update_url, username) {

  //retrieve list of existing searches - if that succeeds, construct modal form
  function create() {

    $.ajaxSetup({
      beforeSend: function(xhr, settings) {
        var csrftoken = $.cookie('csrftoken');
        xhr.setRequestHeader('X-CSRFToken', csrftoken);
      }
    });

    return $.ajax({
      type: 'GET',
      url: list_url + '?fields=name',

    }).pipe(function(data) {

      var message = $(document.createElement('div'));
      message.append($(document.createElement('p')).text('By clicking Save, you will grant supervisor access to this user. Therefore giving this person the opportunity to handle supervisees. You can cancel the operation by clicking Cancel.'));
      var form_group = $(document.createElement('div'));
      form_group.addClass('form-group has-feedback');
      message.append(form_group);
      var input_group = $(document.createElement('div'));
      input_group.addClass('input-group');
      input_group.append($(document.createElement('span')).addClass('input-group-addon').text(username + ' :'));
      var input = $(document.createElement('input')).addClass('form-control');
      input.attr('type', 'text');
      input.attr('placeholder', 'Start typing for auto-completion...');
      if (name) input.val(name.split('/')[1]);
      input_group.append(input);
      glyph = $(document.createElement('span'));
      glyph.addClass('glyphicon glyphicon-ok form-control-feedback');
      glyph.attr('aria-hidden', 'true');
      input_group.append(glyph);
      form_group.append(input_group);

      var available_names = {};
      data.forEach(function(i){available_names[i.username]=i;});
      var available_name_keys = Object.keys(available_names);

      //visual feedback if new or overwriting...
      var help = $(document.createElement('span')).addClass('help');
      form_group.append(help);

      var update_help = function() {
        var currval = input.val();
        if (available_name_keys.indexOf(currval) < 0) {
          help.text('User name is invalid.');
          form_group.removeClass('has-success');
          form_group.addClass('has-error');
          glyph.removeClass('glyphicon-ok');
          glyph.addClass('glyphicon-remove');
        }
        else {
          help.text('');
          form_group.removeClass('has-error');
          form_group.addClass('has-success');
          glyph.removeClass('glyphicon-remove');
          glyph.addClass('glyphicon-ok');
        }
      }
      update_help(); //checks currently set value
      input.on('keyup', update_help);

      //using bootstrap3-typeahead
      input.typeahead({
        source: Object.keys(available_names),
        items: 5, //how many to show at a time
        minLength: 0, //show suggestions even if no text available
        autoSelect: false, //user may want to leave it where it is
        afterSelect: update_help, //updates after the user selected an item
        showHintOnFocus: true, //dropdown as soon as focus on
      });

      BootstrapDialog.show({
        title: '<i class="fa fa-floppy-o fa-lg"> Select a name',
        message: message,
        type: BootstrapDialog.TYPE_PRIMARY,
        buttons: [
        {
          label: 'Cancel',
          cssClass: 'btn-default',
          action: function(the_dialog) {
            the_dialog.close();
            return false;
          },
        },
        {
          label: 'Update',
          cssClass: 'btn-primary',
          action: function(the_dialog) {
            var val = input.val();
            if (!val) {
              BootstrapDialog.alert({
                title: '<i class="fa fa-warning"></i> Error',
                message: 'You must set a username to grant supervisor access',
                type: BootstrapDialog.TYPE_WARNING,
              });
              return false;
            }
            else if (available_name_keys.indexOf(val) < 0) {
              BootstrapDialog.alert({
                title: '<i class="fa fa-warning"></i> Error',
                message: 'You must enter an existing user to grant supervisor access',
                type: BootstrapDialog.TYPE_WARNING,
              });
              return false;
            }
            the_dialog.close();

            if (val) {
              var d = $.ajax({
                type: 'PUT',
                url: update_url,
                data: JSON.stringify({name: val}),
                contentType: "application/json; charset=utf-8",
                dataType: "json"
              });

              d.done(function(data, status) {
                the_dialog.close();
                location.reload();
                return true;
              });

              d.fail(process_error);

            } //val is set
          },
        },
        ],
      });
    });
  }

  return create().fail(process_error);

}
